Package: pmp
Type: Package
Title: Peak Matrix Processing and signal batch correction for
        metabolomics datasets
Version: 1.23.1
Authors@R: c(
    person(
        c("Andris"),
          "Jankevics",
           role=c("aut"),
           email="a.jankevics@bham.ac.uk"),
    person(
        c("Gavin","Rhys"),
          "Lloyd",
           role=c("aut","cre"),
           email="g.r.lloyd@bham.ac.uk"),
    person(
        c("Ralf","Johannes", "Maria"),
          "Weber",
           role=c("aut"),
           email="r.j.weber@bham.ac.uk")
    )
Maintainer@R: c( person( c("Gavin","Rhys"), "Lloyd",
        role=c("aut","cre"), email="g.r.lloyd@bham.ac.uk") )
Description: Methods and tools for (pre-)processing of metabolomics datasets
    (i.e. peak matrices), including filtering, normalisation, missing value
    imputation, scaling, and signal drift and batch effect correction methods.
    Filtering methods are based on: the fraction of missing values (across
    samples or features); Relative Standard Deviation (RSD) calculated from the
    Quality Control (QC) samples; the blank samples. Normalisation methods
    include Probabilistic Quotient Normalisation (PQN) and normalisation to
    total signal intensity. A unified user interface for several commonly used
    missing value imputation algorithms is also provided. Supported methods
    are: k-nearest neighbours (knn), random forests (rf), Bayesian PCA missing
    value estimator (bpca), mean or median value of the given feature and a
    constant small value. The generalised logarithm (glog) transformation
    algorithm is available to stabilise the variance across low and high
    intensity mass spectral features. Finally, this package provides an
    implementation of the Quality Control-Robust Spline Correction (QCRSC)
    algorithm for signal drift and batch effect correction of mass
    spectrometry-based datasets.
License: GPL-3
biocViews: MassSpectrometry, Metabolomics, Software, QualityControl,
        BatchEffect
Depends: R (>= 4.0)
Imports: stats, impute, pcaMethods, missForest, ggplot2, methods,
        SummarizedExperiment, S4Vectors, matrixStats, grDevices,
        reshape2, utils
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.1
Suggests: testthat, covr, knitr, rmarkdown, BiocStyle, gridExtra,
        magick
VignetteBuilder: knitr
Collate: 'checkPeakMatrix.R' 'utils.R' 'data.R' 'filters.R'
        'glog_transformation.R' 'mv_imputation.R' 'normalisation.R'
        'sbc_main.R' 'sbc_methods.R' 'sbc_plot.R'
git_url: https://git.bioconductor.org/packages/pmp
git_branch: devel
git_last_commit: 0f6f235
git_last_commit_date: 2025-11-06
Repository: Bioconductor 3.23
Date/Publication: 2025-11-06
NeedsCompilation: no
Packaged: 2025-11-07 00:14:56 UTC; biocbuild
Author: Andris Jankevics [aut],
  Gavin Rhys Lloyd [aut, cre],
  Ralf Johannes Maria Weber [aut]
Maintainer: Gavin Rhys Lloyd <g.r.lloyd@bham.ac.uk>
