# Generated by roxygen2: do not edit by hand

export(QCRSC)
export(filter_peaks_by_blank)
export(filter_peaks_by_fraction)
export(filter_peaks_by_rsd)
export(filter_samples_by_mv)
export(glog_plot_optimised_lambda)
export(glog_transformation)
export(mv_imputation)
export(normalise_to_sum)
export(pqn_normalisation)
export(processing_history)
export(remove_peaks)
export(sbc_plot)
import(SummarizedExperiment)
import(ggplot2)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,metadata)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(impute,impute.knn)
importFrom(matrixStats,rowAnyMissings)
importFrom(matrixStats,rowCounts)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,rowVars)
importFrom(methods,as)
importFrom(methods,is)
importFrom(missForest,missForest)
importFrom(pcaMethods,pca)
importFrom(reshape2,melt)
importFrom(stats,median)
importFrom(stats,optimise)
importFrom(stats,predict)
importFrom(stats,sd)
importFrom(stats,smooth.spline)
importFrom(stats,var)
importFrom(utils,capture.output)
